-- Kui_Nameplates_Trivial
-- By Michael

local addon = LibStub('AceAddon-3.0'):GetAddon('KuiNameplates')
local mod = addon:NewModule('TrivialUnits', 'AceEvent-3.0')

mod.uiName = 'Trivial units'

-- Variables
local trivUnits = {}

-- Configuration interface
mod.configChangedFuncs = { runOnce = {} }
mod.configChangedFuncs.runOnce.enabled = function(v)
	if v then mod:Enable() else	mod:Disable() end
end
mod.configChangedFuncs.runOnce.trivialUnits = function(v)
	mod:LoadConfig()
end

function mod:GetOptions()
	return {
		enabled = {
            name = 'Enable user defined trivial units',
            desc = 'Enable user defined trivial units',
            width = 'double',
            type = 'toggle',
            order = 1,
            disabled = false
		},
		trivialUnits = {
			name = 'Trivial units names',
			desc = 'Type the name of the trivial mobs. Delimiters "#@\/\\-" can be used to add comments.',
			width = "full",
			type = "input",
			order = 2,
			multiline = 20,
		},
	}
end

function mod:LoadConfig()
	wipe(trivUnits)
	local t = { strsplit("\n",self.db.profile.trivialUnits) }
	for i=1,#t do -- Remove comments: any text starting with #@\/-[ characters.
		local s = strtrim( (strsplit( "#@\\\/\-\[", t[i] )) ) -- Don't remove strsplit extra brackets.
		if strlen(s)>0 then
			trivUnits[s] = true
		end
	end
end

function mod:OnEnable()
	self:LoadConfig()
end

function mod:OnDisable()
	wipe(trivUnits)
end

-- Module initialization
function mod:OnInitialize()
	self.db = addon.db:RegisterNamespace(self.moduleName, {
		profile = {	enabled = false, trivialUnits = {} },
	})
	addon:InitModuleOptions(self)
	self:SetEnabledState(self.db.profile.enabled)
	self:RegisterMessage('KuiNameplates_PostCreate', 'PostCreate')
end

-- Unit is trivial if nameplate scale<1, so we are returning 0.999 for our custom trivial units
local function GetScale(self)
	local scale = self:oldGetScale()
	return scale>=1 and trivUnits[ self.oldName:GetText() or '' ] and 0.999 or scale
end

-- For future versiones of KuiNameplates
local function IsTrivial(self)
    return ( not addon.notrivial ) and ( self.firstChild:GetScale() < 1 or trivUnits[ self.oldName:GetText() or '' ] )
end

function mod:PostCreate(msg, frame)
	if frame.IsTrivial then
		mod.PostCreate = function(self, msg, frame) -- Future versions of KuiNameplates 
			frame.IsTrivial = IsTrivial
		end
	else
		mod.PostCreate = function(self, msg, frame)	-- Current version of KuiNameplates
			local firstChild = frame.firstChild
			firstChild.oldName = frame.oldName
			firstChild.oldGetScale = firstChild.GetScale
			firstChild.GetScale = GetScale
		end	
	end	
	self:PostCreate(msg, frame)
end
