﻿-- ------------------------------------------------------------------------- --
-- Project: Legible Mail         
-- Author:  VincentSDSH				
-- Version: 1.1a - w8.0
-- ------------------------------------------------------------------------- --
-- Command /lm -- loads the configuration UI
-- Command /rl -- reloads the game -- nb: disabled for prod
--                but you're welcome to unrem the line if you want to use it
-- ------------------------------------------------------------------------- --

-- [ Libraries and Modules ] ------------------------------------- --
LegibleMail = LibStub("AceAddon-3.0"):NewAddon("LegibleMail", "AceConsole-3.0")
local LSM   = LibStub("LibSharedMedia-3.0")
-- ------------------------------------- [ Libraries and Modules ] --

-- [ Init and Enable ] ------------------------------------------- --
function LegibleMail:OnInitialize()
  -- Universal ----------------------------- --  
  self.UpdatedForWoWVer = "7.0"
  self.VersionNotes = "Updates for 7.0 Expansion"

  -- Defaults ------------------------------ --
	self.DBDefaults = {
      ["global"] = {
      	["font"]     = "Friz Quadrata TT",
				["fontSize"] = 14
      },
  }
  self.db = LibStub("AceDB-3.0"):New("LegibleMailDB", self.DBDefaults, "Default") 

	-- Options ------------------------------- --
	local LMOpts = {
		type = "group",
		name = "Legible Mail -- Font Replacer Options",
		args = {
			intro = {
				order = 1,
				type = "description",
				fontSize = "medium",
				name = "|cffdeb887Legible Mail lets you set the font for in-game mail.|r",
			},
			introspacer = { -- really needs to be a better way to do this...
				order = 2,
				type = "description",
				name = "  "
			},

			usage = {
				order = 3,
				type = "description",
				fontSize = "medium",
				name = "|cff87d8de*|r|cffdeb887 Best usage is to have a mail item open (sending or reading), type /LM to bring up this panel, and make your mail legible.|r",
			},
			usagespacer = {
				order = 4,
				type = "description",
				name = "  "
			},

			libSM = {
				order = 5,
				type = "description",
				fontSize = "medium",
				name = "|cff87d8de*|r|cffdeb887 Font selections are supplied by libSharedMedia so the available selections will vary with the mods installed.|r",
			},

			font = {
				type = "select",
				name = "Font",
				values = LSM:List("font"),
				get = function(info)					
					for i, v in next, LSM:List("font") do
						if v == LegibleMail.db.global.font then return i end
					end
				end,
				set = function(info, value)
					local list = LSM:List("font")
					LegibleMail.db.global.font = list[value]
					LegibleMail:UpdateMailFrame()
				end,
				disabled = false,
				order = 10,
				width = "normal",
			},
			fontSize = {
				type = "range",
				name = "Font Size",
				min = 6, max = 64, step = 1, -- Legion ups the max font size seems to be 64 for this object
				get = function(info) return LegibleMail.db.global.fontSize end,
				set = function(info, value)
					LegibleMail.db.global.fontSize = value					
					LegibleMail:UpdateMailFrame()
				end,
				disabled = false,
				order = 11,
				width = "normal",
			},
			prefooter = {
				order = 10,
				type = "description",
				fontSize = "medium",
				name = " ",
			},
			footer = {
				order = 101,
				type = "description",
				fontSize = "medium",
				name = "|cffdeb887Legible Mail version |r|cffaede87"..GetAddOnMetadata("LegibleMail", 'Version').." |r|cffdeb887(updated WoW Version |r|cff87afde"..LegibleMail.UpdatedForWoWVer.."|r|cffdeb887)|r",
			},
		},
	}
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("LegibleMail", LMOpts, true)	
	self.Grip = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("LegibleMail", "Legible Mail")
 
  -- Setup Commandline ---------------------- --
  self:RegisterChatCommand("lm", "doLoadConfig")
  -- self:RegisterChatCommand("rl", "doReload") -- REMed for production release
  
  -- Apply the Font
  self:UpdateMailFrame()
end
function LegibleMail:doLoadConfig() InterfaceOptionsFrame_OpenToCategory(LegibleMail.Grip); end -- LegibleMail.panel.name
function LegibleMail:doReload() ReloadUI(); end

function LegibleMail:OnEnable()
	self:UpdateMailFrame() -- set font
end

function LegibleMail:UpdateMailFrame()
	local sdg = self.db.global	
	local fontObject = CreateFont("LegibleMail_Font")
	local isOK = fontObject:SetFont(LSM:Fetch("font", sdg.font), sdg.fontSize, "")	
		
	OpenMailBodyText:SetFontObject(fontObject)		-- READING
	SendMailBodyEditBox:SetFontObject(fontObject)	-- WRITING
	
end